"""
Created on 2022-04-11
@author:刘飞
@description:发布子模块逻辑处理
"""
import datetime
import logging
import zlib

from django.core.paginator import EmptyPage
from django.core.paginator import PageNotAnInteger
from django.core.paginator import Paginator
from django.db.models import F
from rest_framework import serializers

from ..models import Thread
from ..models import ThreadAuth
from ..models import ThreadCategory
from ..models import ThreadClassify
from ..models import ThreadShow
from ..models import ThreadStatistic
from ..models import ThreadTag
from ..models import ThreadTagMapping
from ..serializers import ThreadAuthListSerializer
from ..serializers import ThreadCategoryListSerializer
from ..serializers import ThreadClassifyListSerializer
from ..serializers import ThreadDetailSerializer
from ..serializers import ThreadListSerializer
from ..serializers import ThreadShowListSerializer
from ..serializers import ThreadTagSerializer

log = logging.getLogger()


class ThreadServices:
    def __init__(self):
        pass

    @staticmethod
    def thread_show(request):
        """
        展示类型。用于对前端界面的显示样式进行分类
        """
        thread_show_obj = ThreadShow.objects.all()
        res = ThreadShowListSerializer(thread_show_obj, many=True)
        return res.data, None

    @staticmethod
    def thread_classify(request):
        """
        分类。具体的分类，可以是按行业、兴趣、学科的分类，是主类别下的子分类。
        """
        category_value = request.query_params.get('category_value', None)
        thread_classify_obj = ThreadClassify.objects.all()
        if category_value:
            thread_classify_obj = thread_classify_obj.filter(category_id__value=category_value)
        res = ThreadClassifyListSerializer(thread_classify_obj, many=True)
        return res.data, None

    @staticmethod
    def thread_category(request):
        """
        类别。类似于版块大类的概念，用于圈定信息内容所属的主要类别
        """
        thread_category_obj = ThreadCategory.objects.all()
        res = ThreadCategoryListSerializer(thread_category_obj, many=True)
        return res.data, None

    @staticmethod
    def thread_auth(request):
        """
        访问权限。作者指定允许哪里用户可以访问，例如私有、公开、好友、指定某些人可以访问等。
        """

        thread_auth_obj = ThreadAuth.objects.all()
        res = ThreadAuthListSerializer(thread_auth_obj, many=True)
        return res.data, None

    @staticmethod
    def thread_tag(request):
        """
        标签类型，存放预置标签。
        """
        size = request.query_params.get('size', 10)
        page = request.query_params.get('page', 1)
        thread_tag_obj = ThreadTag.objects.all()
        paginator = Paginator(thread_tag_obj, size)
        try:
            thread_tag_obj = paginator.page(page)
        except PageNotAnInteger:
            thread_tag_obj = paginator.page(1)
        except EmptyPage:
            thread_tag_obj = paginator.page(paginator.num_pages)
        except Exception as e:
            log.error(f'信息主表分页:{str(e)}')
            raise serializers.ValidationError(str(e))
        res = ThreadTagSerializer(thread_tag_obj, many=True)
        data = {'total': paginator.count, 'list': res.data}
        return data, None

    @staticmethod
    def thread_list_read(request):
        """
        信息主表列表读取
        """
        size = request.query_params.get('size', 10)
        page = request.query_params.get('page', 1)
        # category_id = request.query_params.get('category_id')
        category_value = request.query_params.get('category_value')
        classify_id = request.query_params.get('classify_id')
        classify_value = request.query_params.get('classify_value')
        title = request.query_params.get('title')
        content = request.query_params.get('content')
        tag_list = request.query_params.get('tag_list')  # 列表[1,2,3,4]
        tags = request.query_params.get('tags')  # 列表['同城', '圣诞节']查询不用这个
        start_time = request.query_params.get('start_time')
        end_time = request.query_params.get('end_time')
        user_id = request.query_params.get('user_id')
        is_deleted = False
        # 时间格式验证
        try:
            if start_time:
                datetime.datetime.strptime(start_time, "%Y-%m-%d %H:%M:%S")
            if end_time:
                datetime.datetime.strptime(start_time, "%Y-%m-%d %H:%M:%S")
        except ValueError:
            raise serializers.ValidationError(f'时间格式错误:它的格式应该是YYYY-MM-DD HH:MM:SS')
        # 边界检查，不写这行，当size为0时，页面会报分母不能为零
        if int(size) <= 0:
            raise serializers.ValidationError(f'请求每页数量(size)不能为零。')

        keys = 'category_id__value classify_id classify_id__value title__icontains content__icontains create_time__gte create_time__lte is_deleted user_id'.split()
        values = [category_value, classify_id, classify_value, title, content, start_time, end_time, is_deleted,
                  user_id]
        conditions = {k: v for k, v in zip(keys, values) if v or v is False}
        # 根据是否朋友圈预查询出所有[除了朋友圈，其他打乱顺序随机返回]
        if category_value and category_value == 'moment':
            thread_obj = Thread.objects.all()
        else:
            thread_obj = Thread.objects.all().order_by('?')
        # 这里先处理标签查询
        if tag_list:
            try:
                thread_id_list = ThreadTagMapping.objects.filter(tag_id__in=tag_list.split(',')).values_list(
                    'thread_id', flat=True)
                thread_obj = thread_obj.filter(id__in=thread_id_list)
            except ValueError as e:
                log.error(f'信息表标签查询{e}')
                pass

        thread_main_obj = thread_obj.filter(**conditions)
        paginator = Paginator(thread_main_obj, size)

        try:
            thread_main_obj = paginator.page(page)
        except PageNotAnInteger:
            thread_main_obj = paginator.page(1)
        except EmptyPage:
            thread_main_obj = paginator.page(paginator.num_pages)
        except Exception as e:
            log.error(f'信息主表分页:{str(e)}')
            raise serializers.ValidationError(str(e))
        res = ThreadListSerializer(thread_main_obj, many=True)
        data = {'total': paginator.count, 'list': res.data}
        return data, None

    @staticmethod
    def thread_list_create(request):
        """
        信息表新增
        """
        serializer = ThreadListSerializer(data=request.data)
        if not serializer.is_valid(raise_exception=True):
            raise serializers.ValidationError(serializer.errors)
        res = serializer.save()  # 这个返回的是对象，后续处理使用

        # 信息统计表创建一条数据
        ThreadStatistic.objects.create(thread_id=res)

        # 处理信息标签
        tag_list = request.data.get('tag_list', [])
        tags = request.data.get('tags', [])
        for t_id in tag_list:
            t_obj = ThreadTag.objects.filter(id=t_id).first()
            if t_obj:
                conditions = {'thread_id': res, 'tag_id': t_obj}
                ThreadTagMapping.objects.create(**conditions)
        for t_value in tags:
            t_obj = ThreadTag.objects.filter(value=t_value).first()
            if not t_obj:
                t_obj = ThreadTag.objects.create(value=t_value)
            conditions = {'thread_id': res, 'tag_id': t_obj}
            ThreadTagMapping.objects.create(**conditions)
        return None, None

    @staticmethod
    def thread_info_update(request, pk):
        thread_obj = Thread.objects.filter(id=pk, is_deleted=False).first()
        now = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        # 获取到更新前的数据
        old_data_list = [thread_obj.is_deleted,
                         thread_obj.category_id,
                         thread_obj.classify_id,
                         thread_obj.show_id,
                         thread_obj.user_id,
                         thread_obj.auth_id,
                         thread_obj.title,
                         thread_obj.content,
                         thread_obj.has_enroll,
                         thread_obj.has_fee,
                         thread_obj.has_comment,
                         thread_obj.cover,
                         thread_obj.video,
                         thread_obj.photos,
                         thread_obj.files]

        # 更新数据
        serializer = ThreadDetailSerializer(thread_obj, data=request.data)
        if not serializer.is_valid(raise_exception=True):
            raise serializers.ValidationError(serializer.errors)
        res = serializer.save()  # 这个返回的是对象，后续处理使用

        # 处理信息标签
        ThreadTagMapping.objects.filter(thread_id=res).delete()  # 先删除掉原有关联
        tag_list = request.data.get('tag_list', [])
        tags = request.data.get('tags', [])
        for t_id in tag_list:
            t_obj = ThreadTag.objects.filter(id=t_id).first()
            if t_obj:
                conditions = {'thread_id': res, 'tag_id': t_obj}
                ThreadTagMapping.objects.create(**conditions)
        for t_value in tags:
            t_obj = ThreadTag.objects.filter(value=t_value).first()
            if not t_obj:
                t_obj = ThreadTag.objects.create(value=t_value)
            conditions = {'thread_id': res, 'tag_id': t_obj}
            ThreadTagMapping.objects.create(**conditions)

        # 更新成功记录日志
        # 获取更新后数据集
        new_data_list = [res.is_deleted,
                         res.category_id,
                         res.classify_id,
                         res.show_id,
                         res.user_id,
                         res.auth_id,
                         res.title,
                         res.content,
                         res.has_enroll,
                         res.has_fee,
                         res.has_comment,
                         res.cover,
                         res.video,
                         res.photos,
                         res.files]
        logs = thread_obj.logs if thread_obj.logs else []
        update_data = {"update_time": now}
        keys = ['is_deleted', 'category_id', 'classify_id', 'show_id', 'user_id', 'auth_id', 'title', 'content',
                'has_enroll', 'has_fee', 'has_comment', 'cover', 'video', 'photos', 'files']
        for k, o, n in zip(keys, old_data_list, new_data_list):
            if zlib.crc32(str(o).encode('utf8')) != zlib.crc32(str(n).encode('utf8')):
                update_data[k] = {"before": o, "after": n}
        # logs = [{"update_time": "2022-05-05 16:02:00", "title": {"before": "修改前", "after": "修改后"}}]
        logs.append(update_data)
        # Thread.objects.filter(id=res.id).update(logs=logs)
        res.logs = logs
        res.save()
        return None, None

    @staticmethod
    def thread_main_info(pk):
        thread_obj = Thread.objects.filter(id=pk, is_deleted=False).first()
        # 信息统计表更新数据
        if thread_obj:
            ThreadStatistic.objects.filter(thread_id=thread_obj).update(views=F('views') + 1)
        res = ThreadDetailSerializer(thread_obj)
        return res.data, None

    @staticmethod
    def thread_main_delete(request, pk):
        thread_obj = Thread.objects.filter(id=pk, is_deleted=False).first()
        thread_obj.is_deleted = True
        thread_obj.save()
        return None, None

    # 已拆分上面三个子方法
    # @staticmethod
    # def thread_main_detail(request, pk):
    #     thread_obj = Thread.objects.filter(id=pk, is_deleted=False).first()
    #     if not thread_obj:
    #         raise serializers.ValidationError('资源不存在')
    #     if request.method == 'GET':
    #         # 信息统计表更新数据
    #         ThreadStatistic.objects.filter(thread_id=thread_obj).update(views=F('views') + 1)
    #         res = ThreadDetailSerializer(thread_obj)
    #         return res.data, None
    #     elif request.method == 'PUT':
    #         now = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
    #         # 获取到更新前的数据
    #         old_data_list = [thread_obj.is_deleted,
    #                          thread_obj.category_id,
    #                          thread_obj.classify_id,
    #                          thread_obj.show_id,
    #                          thread_obj.user_id,
    #                          thread_obj.auth_id,
    #                          thread_obj.title,
    #                          thread_obj.content,
    #                          thread_obj.has_enroll,
    #                          thread_obj.has_fee,
    #                          thread_obj.has_comment,
    #                          thread_obj.cover,
    #                          thread_obj.video,
    #                          thread_obj.photos,
    #                          thread_obj.files]
    #
    #         # 更新数据
    #         serializer = ThreadDetailSerializer(thread_obj, data=request.data)
    #         if not serializer.is_valid(raise_exception=True):
    #             raise serializers.ValidationError(serializer.errors)
    #         res = serializer.save()  # 这个返回的是对象，后续处理使用
    #
    #         # 处理信息标签
    #         ThreadTagMapping.objects.filter(thread_id=res).delete()  # 先删除掉原有关联
    #         tag_list = request.data.get('tag_list', [])
    #         tags = request.data.get('tags', [])
    #         for t_id in tag_list:
    #             t_obj = ThreadTag.objects.filter(id=t_id).first()
    #             if t_obj:
    #                 conditions = {'thread_id': res, 'tag_id': t_obj}
    #                 ThreadTagMapping.objects.create(**conditions)
    #         for t_value in tags:
    #             t_obj = ThreadTag.objects.filter(value=t_value).first()
    #             if not t_obj:
    #                 t_obj = ThreadTag.objects.create(value=t_value)
    #             conditions = {'thread_id': res, 'tag_id': t_obj}
    #             ThreadTagMapping.objects.create(**conditions)
    #
    #         # 更新成功记录日志
    #         # 获取更新后数据集
    #         new_data_list = [res.is_deleted,
    #                          res.category_id,
    #                          res.classify_id,
    #                          res.show_id,
    #                          res.user_id,
    #                          res.auth_id,
    #                          res.title,
    #                          res.content,
    #                          res.has_enroll,
    #                          res.has_fee,
    #                          res.has_comment,
    #                          res.cover,
    #                          res.video,
    #                          res.photos,
    #                          res.files]
    #         logs = thread_obj.logs if thread_obj.logs else []
    #         update_data = {"update_time": now}
    #         keys = ['is_deleted', 'category_id', 'classify_id', 'show_id', 'user_id', 'auth_id', 'title', 'content',
    #                 'has_enroll', 'has_fee', 'has_comment', 'cover', 'video', 'photos', 'files']
    #         for k, o, n in zip(keys, old_data_list, new_data_list):
    #             if zlib.crc32(str(o).encode('utf8')) != zlib.crc32(str(n).encode('utf8')):
    #                 update_data[k] = {"before": o, "after": n}
    #         # logs = [{"update_time": "2022-05-05 16:02:00", "title": {"before": "修改前", "after": "修改后"}}]
    #         logs.append(update_data)
    #         # Thread.objects.filter(id=res.id).update(logs=logs)
    #         res.logs = logs
    #         res.save()
    #         return None, None
    #     elif request.method == 'DELETE':
    #         thread_obj.is_deleted = True
    #         thread_obj.save()
    #         return None, None



"""
Created on 2022-04-11
@description:刘飞
@description:发布子模块逻辑分发
"""
from rest_framework.views import APIView
from xj_user.utils.custom_authorization import CustomAuthentication

from ..services.theard_v1 import ThreadServices
from ..utils.custom_authentication_wrapper import authentication_wrapper
from ..utils.custom_response import util_response

t = ThreadServices()


class ShowListAPIView(APIView):
    """
    get:展示类型列表
    """

    authentication_classes = (CustomAuthentication,)

    def get(self, request, *args, **kwargs):
        data, error_text = t.thread_show(request)
        return util_response(data=data)


class ClassifyListAPIView(APIView):
    """
    get:分类列表
    """

    # authentication_classes = (CustomAuthentication,)

    def get(self, request, *args, **kwargs):
        data, error_text = t.thread_classify(request)
        return util_response(data=data)


class CategoryListAPIView(APIView):
    """
    get:类别列表
    """

    # authentication_classes = (CustomAuthentication,)

    def get(self, request, *args, **kwargs):
        data, error_text = t.thread_category(request)
        return util_response(data=data)


class AuthListAPIView(APIView):
    """
    get:访问权限列表
    """

    # authentication_classes = (CustomAuthentication,)

    def get(self, request, *args, **kwargs):
        data, error_text = t.thread_auth(request)
        return util_response(data=data)


class TagListAPIView(APIView):
    """
    get:标签列表
    """

    # authentication_classes = (CustomAuthentication,)

    def get(self, request):
        data, error_text = t.thread_tag(request)
        return util_response(data=data)


class ThreadListAPIView(APIView):
    """
    get: 信息表列表
    post: 信息表新增
    """

    # authentication_classes = (CustomAuthentication,)

    # @authentication_wrapper
    def get(self, request, *args, **kwargs):
        data, error_text = t.thread_list_read(request)
        return util_response(data=data)

    @authentication_wrapper
    def post(self, request, *args, **kwargs):
        request.data['user_id'] = request.user.get('user_id', None)
        data, error_text = t.thread_list_create(request)
        return util_response(data=data)


class ThreadDetailAPIView(APIView):
    """
    get:信息表详情
    put:信息表编辑
    delete:信息表删除【假删除】
    """

    # authentication_classes = (CustomAuthentication,)

    def get(self, request, pk, *args, **kwargs):
        data, error_text = t.thread_main_info(request, pk)
        return util_response(data=data)

    @authentication_wrapper
    def put(self, request, pk, *args, **kwargs):
        request.data['user_id'] = request.user.get('user_id', None)
        data, error_text = t.thread_info_update(request, pk)
        return util_response(data=data)

    @authentication_wrapper
    def delete(self, request, pk, *args, **kwargs):
        data, error_text = t.thread_main_delete(request, pk)
        return util_response(data=data)



"""
Created on 2022-04-11
@description:刘飞
@description:发布子模块逻辑分发
"""
from rest_framework.views import APIView

from ..services.thread_list_service import ThreadListService
# from ..utils.custom_authentication_wrapper import authentication_wrapper
from ..utils.custom_response import util_response


class ThreadListAPIView(APIView):
    """
    get: 信息表列表
    post: 信息表新增
    """

    # @authentication_wrapper
    def get(self, request, *args, **kwargs):
        params = request.query_params
        size = request.GET.get('size', 20)
        if int(size) > 100:
            return util_response(msg='每一页不可以超过100条', err=40225)
        data, error_text = ThreadListService.list(params)
        return util_response(data=data)



# encoding: utf-8
"""
@project: djangoModel->extend_service
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 扩展服务
@created_time: 2022/7/29 15:14
"""

from ..models import ThreadExtendField, Thread
from ..utils.model_handle import *


class InputExtend:
    thread_extend_filed = None

    def __init__(self, form_data, need_all_field=False):
        """
        :param form_data: 表单
        :param need_all_field: 是否需要全部的扩展字段（查询的时候会用到）
        """
        self.form_data = form_data
        if form_data.get('classify_id', None):
            self.form_data['classify_id_id'] = self.form_data.pop('classify_id')
        if form_data.get('category_id', None):
            self.form_data['category_id_id'] = self.form_data.pop('category_id')

        if need_all_field:
            self.thread_extend_filed = parse_model(ThreadExtendField.objects.all())
            self.thread_extend_filed = {item["field"]: item["extend_field"] for item in self.thread_extend_filed if self.thread_extend_filed}
        else:
            # 新增或者修改的时候
            if "id" in self.form_data.keys():  # 修改时候：传了id,没有传classify_id
                thread = Thread.objects.filter(id=self.form_data.get('id')).first()
                self.thread_extend_filed = {
                    item["field"]: item["extend_field"]
                    for item in ThreadExtendField.objects.filter(classify_id=thread.classify_id).values('field', 'extend_field')
                }
            if self.form_data.get('category_id_id', None):
                self.thread_extend_filed = {
                    item["field"]: item["extend_field"]
                    for item in ThreadExtendField.objects.filter(classify_id=self.form_data.get('category_id_id')).values('field', 'extend_field')
                }

    # 请求参数转换
    def transform_param(self):
        # 没有定义扩展映射直接返回，不进行扩展操作
        if self.thread_extend_filed is None:
            return self.form_data, None
        extend_data = {self.thread_extend_filed[k]: self.form_data.pop(k) for k, v in self.form_data.copy().items() if k in self.thread_extend_filed.keys()}
        return self.form_data, extend_data



"""
Created on 2022-04-11
@description:刘飞
@description:发布子模块单挑数据删除/修改/详情
"""
from rest_framework.views import APIView

from xj_thread.services.thread_item_service import ThreadItemService
from xj_thread.utils.custom_authentication_wrapper import authentication_wrapper
from xj_thread.utils.model_handle import parse_data
from ..utils.custom_response import util_response

# class ThreadItemAPIView(APIView):
#     """
#     get: 信息表列表
#     post: 信息表新增
#     """
#
#     # @authentication_wrapper
#     def get(self, request, *args, **kwargs):
#         """查看单条信息"""
#         params = request.query_params
#         data, error_text = ThreadListService.list(params)
#         return util_response(data=data)
#
#     @authentication_wrapper
#     def post(self, request, *args, **kwargs):
#         """新增信息"""
#         request.data['user_id'] = request.user.get('user_id', None)
#         data, error_text = t.thread_list_create(request)
#         return util_response(data=data)

item_service = ThreadItemService()


class ThreadItemlAPI(APIView):
    """单挑信息处理，查，改，删"""

    def get(self, request, pk, *args, **kwargs):
        """信息表详情"""
        if not pk:
            return util_response(msg="非法请求", err=2554)
        data, error_text = item_service.detail(pk)

        if not error_text:
            return util_response(data=data)
        return util_response(err=47767, msg=error_text)

    @authentication_wrapper
    def put(self, request, pk, *args, **kwargs):
        """信息表编辑"""
        if not pk:
            return util_response(msg="非法请求", err=2554)
        form_data = parse_data(request=request, only_field=[
            'title', 'content', 'summary', 'ip', 'has_enroll', 'video', 'files', 'logs', 'price', 'author', 'classify_id', 'category_id',
            'auth_id', 'cover', 'show_id', 'hot_loop', 'hot_tip'
        ])
        data, error_text = item_service.edit(form_data, pk)
        if not error_text:
            return util_response()
        return util_response(err=47767, msg=error_text)

    @authentication_wrapper
    def delete(self, request, pk, *args, **kwargs):
        data, error_text = item_service.delete(pk)
        print(error_text)
        if not error_text:
            return util_response()
        return util_response(err=47767, msg=error_text)



from django.contrib import admin

from .models import Thread, ThreadStatistic
from .models import ThreadExtendData
from .models import ThreadExtendField
from .models import ThreadImageAuth, ThreadResource
from .models import ThreadShow, ThreadClassify, ThreadCategory, ThreadAuth
from .models import ThreadTag, ThreadTagMapping


@admin.register(ThreadShow)
class ThreadShowAdmin(admin.ModelAdmin):
    list_display = ('id', 'value', 'config', 'description')
    search_fields = ('id', 'value', 'config')
    fields = ('value', 'config', 'description')


@admin.register(ThreadClassify)
class ThreadClassifyAdmin(admin.ModelAdmin):
    list_display = ('id', 'value', 'show_id', 'category_id', 'description', 'icon')
    search_fields = ('id', 'value', 'show_id', 'category_id')
    fields = ('value', 'show_id', 'category_id', 'description', 'icon')


@admin.register(ThreadCategory)
class ThreadCategoryAdmin(admin.ModelAdmin):
    list_display = ('id', 'value', 'description')
    search_fields = ('id', 'value')
    fields = ('value', 'description')


@admin.register(ThreadAuth)
class ThreadAuthAdmin(admin.ModelAdmin):
    list_display = ('id', 'value')
    search_fields = ('id', 'value')
    fields = ('value',)


@admin.register(Thread)
class ThreadAdmin(admin.ModelAdmin):
    list_display = ('id', 'is_deleted', 'category_id', 'classify_id', 'show_id', 'user_id', 'auth_id',
                    'author', 'title', 'short_content', 'ip', 'has_enroll', 'has_fee', 'has_comment',
                    'short_cover', 'short_photos', 'short_video', 'short_files', 'short_logs', 'more')
    search_fields = ('id', 'is_deleted', 'category_id', 'classify_id', 'show_id', 'user_id', 'auth_id',
                     'title', 'content', 'ip', 'has_enroll', 'has_fee', 'has_comment',
                     'cover', 'photos', 'video', 'files', 'logs')
    fields = ('id', 'is_deleted', 'category_id', 'classify_id', 'show_id', 'user_id', 'auth_id',
              'author', 'title', 'content', 'ip', 'has_enroll', 'has_fee', 'has_comment',
              'cover', 'photos', 'video', 'files', 'logs', 'more')
    readonly_fields = ('id', )  # 只读
    list_per_page = 10  # 每页显示10条
    ordering = ['-update_time']  # 排序

    # class Media:
    #     css = {
    #         'all': (
    #             '/css/fancy.css',
    #         )
    #     }


@admin.register(ThreadExtendData)
class ThreadExtendDataAdmin(admin.ModelAdmin):
    list_display = ('thread_id', 'field_1', 'field_2', 'field_3', 'field_4', 'field_5', 'field_6', 'field_7')
    search_fields = ('thread_id', 'field_1', 'field_2', 'field_3', 'field_4', 'field_5', 'field_6', 'field_7')
    fields = ('thread_id', 'field_1', 'field_2', 'field_3', 'field_4', 'field_5', 'field_6', 'field_7')


@admin.register(ThreadExtendField)
class ThreadExtendFieldAdmin(admin.ModelAdmin):
    list_display = ('classify_id', 'field', 'extend_field','value', 'type', 'unit')
    search_fields = ('classify_id', 'field', 'extend_field','value', 'type', 'unit')
    fields = ('classify_id', 'field', 'extend_field','value', 'type', 'unit')


@admin.register(ThreadTag)
class ThreadTagAdmin(admin.ModelAdmin):
    list_display = ('id', 'value')
    search_fields = ('id', 'value')
    fields = ('value',)


@admin.register(ThreadTagMapping)
class ThreadTagMappingAdmin(admin.ModelAdmin):
    list_display = ('id', 'thread_id', 'tag_id')
    search_fields = ('id', 'thread_id', 'tag_id')
    fields = ('thread_id', 'tag_id')


@admin.register(ThreadImageAuth)
class ThreadImageAuthAdmin(admin.ModelAdmin):
    list_display = ('id', 'value')
    search_fields = ('id', 'value')
    fields = ('value',)


@admin.register(ThreadResource)
class ThreadImageAdmin(admin.ModelAdmin):
    list_display = (
        'id', 'name', 'url', 'filename', 'filetype', 'image_auth_id', 'price', 'snapshot', 'format', 'logs', 'user_id')
    search_fields = (
        'id', 'name', 'url', 'filename', 'filetype', 'image_auth_id', 'price', 'snapshot', 'format', 'logs', 'user_id')
    fields = (
        'id', 'name', 'url', 'filename', 'filetype', 'image_auth_id', 'price', 'snapshot', 'format', 'logs', 'user_id')


@admin.register(ThreadStatistic)
class ThreadStatisticAdmin(admin.ModelAdmin):
    list_display = (
        'id', 'thread_id', 'flag_classifies', 'flag_weights', 'weight', 'views', 'plays', 'comments', 'likes',
        'favorite', 'shares',
    )
    search_fields = (
        'id', 'thread_id', 'flag_classifies', 'flag_weights', 'weight', 'views', 'plays', 'comments', 'likes',
        'favorite', 'shares',
    )



# Register your models here.
# admin.site.register(ThreadShow, ThreadShowAdmin)
# admin.site.register(ThreadClassify, ThreadClassifyAdmin)
# admin.site.register(ThreadCategory, ThreadCategoryAdmin)
# admin.site.register(ThreadAuth, ThreadAuthAdmin)

# admin.site.register(Thread, ThreadAdmin)

# admin.site.register(ThreadExtendData, ThreadExtendDataAdmin)
# admin.site.register(ThreadExtendField, ThreadExtendFieldAdmin)

# admin.site.register(ThreadTag, ThreadTagAdmin)
# admin.site.register(ThreadTagMapping, ThreadTagMappingAdmin)

# admin.site.register(ThreadImageAuth, ThreadImageAuthAdmin)
# admin.site.register(ThreadResource, ThreadImageAdmin)
# admin.site.site_header = 'msa一体化管理后台'
# admin.site.site_title = 'msa一体化管理后台'


# encoding: utf-8
"""
@project: djangoModel->thread_v2
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis:
@created_time: 2022/7/29 15:11
"""
import datetime
import logging

from django.core.paginator import Paginator
from rest_framework import serializers

from ..models import Thread
from ..models import ThreadTagMapping
from ..serializers import ThreadListSerializer
from ..utils.custom_response import util_response

log = logging.getLogger()


# 信息服务CURD(支持扩展字段配置)
class ThreadListService:
    @staticmethod
    def list(params):
        page = params.get('page', 1)
        size = params.get('size', 20)

        tag_id_list = params.get('tag_id_list') if params.get('tag_id_list') else None  # 列表[1,2,3,4]
        # tag_value_list = params.get('tags', '').split(',')  # 列表['同城', '圣诞节']查询不用这个

        exclude_category_list = params.get('exclude_categorys').split(',') if params.get('exclude_categorys') else None

        # 边界检查：时间格式验证
        start_time = params.get('start_time')
        end_time = params.get('end_time')
        try:
            if start_time:
                datetime.datetime.strptime(start_time, "%Y-%m-%d %H:%M:%S")
            if end_time:
                datetime.datetime.strptime(start_time, "%Y-%m-%d %H:%M:%S")
        except ValueError:
            raise serializers.ValidationError(f'时间格式错误:它的格式应该是YYYY-MM-DD HH:MM:SS')

        # 允许进行过渡的字段条件
        conditions = {
            "category_id": params.get('category_id'),
            "category_id__value": params.get('category_value'),
            "classify_id": params.get('classify_id'),
            "classify_id__value": params.get('classify_value'),
            "title__icontains": params.get('title'),
            "content__icontains": params.get('content'),
            "create_time__gte": params.get('start_time'),
            "create_time__lte": params.get('end_time'),
            "user_id": params.get('user_id'),
            "is_deleted": False,
        }
        conditions = {k: v for k, v in conditions.items() if v or v is False}
        # print("> conditions:", conditions)

        # 边界检查，不写这行，当size为0时，页面会报分母不能为零
        if int(size) <= 0:
            raise serializers.ValidationError(f'请求每页数量(size)不能为零。')

        # 另一个条件查询的生成语句
        # keys = 'category_id__value classify_id classify_id__value title__icontains content__icontains create_time__gte create_time__lte is_deleted user_id'.split()
        # values = [category_value, classify_id, classify_value, title, content, start_time, end_time, is_deleted, user_id]
        # conditions = {k: v for k, v in zip(keys, values) if v or v is False}

        # 开始按过滤条件
        try:
            thread_set = Thread.objects.filter(**conditions)
        except Exception as e:
            return util_response(err=7557, msg=e.__str__())

        # 指定不需要过滤的类别字段
        if exclude_category_list:
            thread_set = thread_set.exclude(category_id__in=exclude_category_list)
        count = thread_set.count()
        # 这里先处理标签查询
        if tag_id_list:
            try:
                thread_id_list = ThreadTagMapping.objects.filter(tag_id__in=tag_id_list).values_list('thread_id', flat=True)  # flat转列表形式
                thread_set = thread_set.filter(id__in=thread_id_list)
            except ValueError as e:
                log.error(f'信息表标签查询{e}')

        # 分页数据
        page_set = ThreadListSerializer(Paginator(thread_set, size).get_page(page), many=True)
        final_res = page_set.data

        for it in final_res:
            it.update(it.pop('statistic'))
            it.update(it.pop('thread_extends'))

        return {'data': final_res, 'size': int(size), 'page': int(page), 'count': count}, None



# encoding: utf-8
"""
@project: djangoModel->thread_v2
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis:
@created_time: 2022/7/29 15:11
"""

from django.db import transaction

from xj_thread.serializers import ThreadDetailSerializer
from xj_thread.services.thread_statistic_service import StatisticsService
from .extend_service import InputExtend
from ..models import Thread
from ..models import ThreadExtendData
from ..utils.custom_response import util_response
from ..utils.model_handle import parse_model


# 信息服务CURD(支持扩展字段配置)  V2版本
class ThreadItemService:
    @staticmethod
    def detail(pk):
        """获取信息内容"""
        thread_obj = Thread.objects.filter(id=pk, is_deleted=False).first()
        if thread_obj:  # 信息统计表更新数据
            StatisticsService.increment(thread_id=thread_obj.id, tag='views', step=1)

        res_set = dict(ThreadDetailSerializer(thread_obj).data)
        # 扁平化数据
        res_set.update(res_set.pop('statistic'))
        res_set.update(res_set.pop('thread_extends'))
        return res_set, 0

    @staticmethod
    def edit(form_data, pk):
        extend_service = InputExtend(form_data)
        # 扩展字段与主表字段拆分
        form_data, extend_form_data = extend_service.transform_param()
        # 开启事务，防止脏数据
        with transaction.atomic():
            save_id = transaction.savepoint()
            try:
                # 主表修改
                main_res = Thread.objects.filter(id=pk)
                if not main_res:
                    return util_response(err=5547, msg="数据不存在，无法进行修改")
                main_res.update(**form_data)
                # 扩展表修改或者创建
                if extend_form_data:
                    extend_res = ThreadExtendData.objects.filter(thread_id=pk)
                    if extend_res:
                        extend_res.update(**extend_form_data)
                    else:
                        form_data['thread_id'] = pk
                        extend_obj = ThreadExtendData(**form_data)
                        extend_obj.save()

                transaction.savepoint_commit(save_id)
            except Exception as e:
                transaction.rollback()
                return None, str(e)
        return None, None

    @staticmethod
    def delete(id):
        main_res = Thread.objects.filter(id=id, is_deleted=0)
        if not main_res:
            return None, "数据不存在，无法进行修改"
        main_res.update(is_deleted=1)
        return None, None

    @staticmethod
    def select_extend(id):
        """单独查询 查询扩展字段"""
        return util_response(parse_model(ThreadExtendData.objects.filter(id=id)))



# encoding: utf-8
"""
@project: djangoModel->thread_add
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 信息添加接口
@created_time: 2022/8/8 13:36
"""
from rest_framework.views import APIView

from xj_thread.services.thread_add_service import ThreadAddService
from xj_thread.utils.model_handle import parse_data, util_response
from xj_thread.validator import ThreadAddValidator


class ThreadAdd(APIView):
    def post(self, request):
        # 字段解析与字段筛选
        only_field = [
            'title', 'content', 'summary', 'ip', 'has_enroll', 'video', 'files', 'logs', 'price', 'author', 'classify_id', 'category_id',
            'auth_id', 'cover', 'show_id', 'hot_loop', 'hot_tip'
        ]
        params = parse_data(request, only_field)
        # # 表单初步验证
        validator = ThreadAddValidator(params)
        is_pass, error = validator.validate()
        if not is_pass:
            return util_response(err=4022, msg=error)
        # 插入服务
        data, err_txt = ThreadAddService.add(params)
        if not err_txt:
            return util_response()
        return util_response(err=47767, msg=err_txt)



# encoding: utf-8
"""
@project: djangoModel->api_v2
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 扩展第二版
@created_time: 2022/7/29 15:25
"""
from rest_framework.views import APIView

from ..services.thread_v2 import StatisticsService, ThreadNewServices
from ..utils.custom_authentication_wrapper import authentication_wrapper
from ..utils.model_handle import *


# 计数统计API
class ThreadStaticAPIView(APIView):
    # 单条自增
    def post(self, request):
        thread_id = request.data.get('thread_id', None)
        tag = request.data.get('tag', None)
        step = request.data.get('step', None)
        tag_list = ['step', 'views', 'plays', 'comments', 'likes', 'favorite', 'shares']
        if not thread_id or not tag in tag_list:
            return StatisticsService.response(use_in_service=False, err=2045, msg="参数错误")
        return StatisticsService.increment(thread_id, tag, step)

    # 多字段自增
    def put(self, request):
        thread_id = request.data.get('thread_id', None)
        if not thread_id:
            return StatisticsService.response(use_in_service=False, err=2045, msg="参数错误")
        form_data = parse_data(request)
        del form_data['thread_id']
        return StatisticsService.increments(thread_id, form_data)


# 统计服务V2
class ThreadAPIView(APIView):
    def get(self, request):
        """列表"""
        return ThreadNewServices.select(request)

    def get_extend(self, request):
        """获取单条的扩展信息"""
        id = request.data.get('id', None)
        if id is None:
            return util_response(err=4015, msg="ID必填")
        return ThreadNewServices.select_extend(id)

    def select_detail(self):
        # 查看信息详情
        return ThreadNewServices.select_detail(self)

    @authentication_wrapper
    def post(self, request):
        """新增"""
        return ThreadNewServices.create(request)

    @authentication_wrapper
    def put(self, request):
        """修改"""
        return ThreadNewServices.update(request)

    @authentication_wrapper
    def delete(self, request):
        """删除"""
        thread_id = request.data.get('id', None)
        if not thread_id:
            return util_response(msg="参数错误", err=2045)
        return ThreadNewServices.delete(id=thread_id)



# encoding: utf-8
"""
@project: djangoModel->thread_add
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 信息添加接口服务
@created_time: 2022/8/8 13:36
"""
from django.db import transaction

from xj_thread.models import ThreadExtendData, Thread, ThreadStatistic
from xj_thread.services.extend_service import InputExtend


class ThreadAddService:
    @staticmethod
    def add(params):
        # 扩展字段与主表字段拆分
        extend_service = InputExtend(params)
        form_data, extend_form_data = extend_service.transform_param()
        # 开启事务，防止脏数据
        with transaction.atomic():
            save_id = transaction.savepoint()
            try:
                instance = Thread.objects.create(**form_data)
                statistic_obj = ThreadStatistic(thread_id_id=instance.id)
                statistic_obj.save()
                if extend_form_data:  # 如果传扩展字段插入
                    extend_form_data['thread_id_id'] = instance.id
                    ThreadExtendData.objects.create(**extend_form_data)
                transaction.savepoint_commit(save_id)
            except Exception as e:
                transaction.savepoint_rollback(save_id)
                return None, str(e)
            return None, None



"""
Created on 2022-04-11
@author:刘飞
@description:发布子模块路由分发
"""
from django.conf.urls import url
from django.urls import re_path

# from .apis.apis_v1 import ThreadListAPIView, ThreadDetailAPIView, AuthListAPIView, CategoryListAPIView, ClassifyListAPIView, ShowListAPIView, TagListAPIView
from xj_thread.apis.thread_add import ThreadAdd
from xj_thread.apis.thread_item import ThreadItemlAPI
from xj_thread.apis.thread_list import ThreadListAPIView
from xj_thread.apis.thread_other_list import AuthListAPIView, CategoryListAPIView, ClassifyListAPIView, ShowListAPIView, TagListAPIView
from xj_thread.apis.thread_static import ThreadStaticAPIView

# 应用名称
# app_name = 'thread'

urlpatterns = [
    re_path(r'^list/?$', ThreadListAPIView.as_view(), name='list'),  # 信息列表/新增
    re_path(r'^add/?$', ThreadAdd.as_view(), name='list'),  # 信息列表/新增
    re_path(r'^item/(?P<pk>\d+)/?$', ThreadItemlAPI.as_view(), name='detail'),  # 信息单挑操作：详情/编辑/删除
    # 列表 信息相关
    url(r'^auth_list/?$', AuthListAPIView.as_view(), name='auth_list'),  # 权限列表
    url(r'^category_list/?$', CategoryListAPIView.as_view(), name='category_list'),  # 类别列表
    url(r'^classify_list/?$', ClassifyListAPIView.as_view(), name='classify_list'),  # 分类列表
    url(r'^show_list/?$', ShowListAPIView.as_view(), name='show_list'),  # 展示类型列表
    url(r'^tag_list/?$', TagListAPIView.as_view(), name='tag_list'),  # 展示类型列表
    url(r'^statistic/?$', ThreadStaticAPIView.as_view(), name='tag_list'),  # 计数统计，前端埋点接口
]



import socket

from DjangoUeditor.models import UEditorField

from django.db import models
from django.utils import timezone

# from apps.user.models import User
# from apps.user.models import BaseInfo

hostname = socket.gethostname()
my_ip_addr = socket.gethostbyname(hostname)


# 展示类型。用于对前端界面的显示样式进行分类
class ThreadShow(models.Model):
    class Meta:
        db_table = 'thread_show'
        verbose_name_plural = '13. 展示类型表 (样式)'

    id = models.AutoField(verbose_name='展示类型ID', primary_key=True)
    value = models.CharField(verbose_name='值', max_length=50)
    config = models.JSONField(verbose_name='前端配置', blank=True, null=True, default=list)  # 用于存放前端自定义的界面或样式相关的配置数据
    description = models.CharField(verbose_name='描述', max_length=255, blank=True, null=True)

    def __str__(self):
        return f"{self.value}"


# 类别。类似于版块大类的概念，用于圈定信息内容所属的主要类别
class ThreadCategory(models.Model):
    class Meta:
        db_table = 'thread_category'
        verbose_name_plural = '11. 类别表 (页面类别)'

    id = models.AutoField(verbose_name='ID', primary_key=True)
    value = models.CharField(verbose_name='值', max_length=50)
    description = models.CharField(verbose_name='描述', max_length=255, blank=True, null=True)

    def __str__(self):
        return f"{self.value}"


class ThreadClassify(models.Model):
    """
    @brief 分类。具体的分类，可以是按行业、兴趣、学科的分类，是主类别下的子分类。
    @note 考虑到多语言翻译的问题，不需要写接口，由运维在后台添加
    """

    class Meta:
        db_table = 'thread_classify'
        verbose_name_plural = '12. 分类表 (行业分类)'

    id = models.AutoField(verbose_name='分类ID', primary_key=True)
    # key = models.CharField(verbose_name='分类关键字', max_length=50, blank=True, null=True)
    value = models.CharField(verbose_name='分类', max_length=50, unique=True)
    show_id = models.ForeignKey(verbose_name='默认展示ID', to=ThreadShow, db_column='show_id', related_name='+',
                                on_delete=models.DO_NOTHING, default=1)
    description = models.CharField(verbose_name='描述', max_length=255, blank=True, null=True)
    category_id = models.ForeignKey(verbose_name='父类别', to=ThreadCategory, db_column='category_id', related_name='+',
                                    on_delete=models.DO_NOTHING, blank=True, null=True)
    icon = models.CharField(verbose_name='图标', max_length=255, blank=True, null=True)

    def __str__(self):
        return f"{self.value}"


# 访问权限。作者指定允许哪里用户可以访问，例如私有、公开、好友、指定某些人可以访问等。
class ThreadAuth(models.Model):
    class Meta:
        db_table = 'thread_auth'
        verbose_name_plural = '权限类型表'

    id = models.AutoField(verbose_name='ID', primary_key=True)
    value = models.CharField(verbose_name='值', max_length=50)

    def __str__(self):
        return f"{self.value}"


# 扩展字段数据表。用于扩展一些自定义的版块功能的数据
class ThreadExtendData(models.Model):
    class Meta:
        db_table = 'thread_extend_data'
        verbose_name_plural = '扩展字段数据表'

    # id = models.AutoField(verbose_name='信息ID', primary_key=True)
    thread_id = models.OneToOneField('Thread',  related_name="thread_extend_data", db_column='thread_id', primary_key=True, on_delete=models.DO_NOTHING)
    field_1 = models.CharField(verbose_name='1', max_length=255, blank=True, null=True)
    field_2 = models.CharField(verbose_name='2', max_length=255, blank=True, null=True)
    field_3 = models.CharField(verbose_name='3', max_length=255, blank=True, null=True)
    field_4 = models.CharField(verbose_name='4', max_length=255, blank=True, null=True)
    field_5 = models.CharField(verbose_name='5', max_length=255, blank=True, null=True)
    field_6 = models.CharField(verbose_name='6', max_length=255, blank=True, null=True)
    field_7 = models.CharField(verbose_name='7', max_length=255, blank=True, null=True)
    field_8 = models.CharField(verbose_name='8', max_length=255, blank=True, null=True)
    field_9 = models.CharField(verbose_name='9', max_length=255, blank=True, null=True)
    field_10 = models.CharField(verbose_name='10', max_length=255, blank=True, null=True)
    field_11 = models.CharField(verbose_name='11', max_length=255, blank=True, null=True)
    field_12 = models.CharField(verbose_name='12', max_length=255, blank=True, null=True)
    field_13 = models.CharField(verbose_name='13', max_length=255, blank=True, null=True)
    field_14 = models.CharField(verbose_name='14', max_length=255, blank=True, null=True)
    field_15 = models.CharField(verbose_name='15', max_length=255, blank=True, null=True)
    field_16 = models.CharField(verbose_name='16', max_length=255, blank=True, null=True)
    field_17 = models.CharField(verbose_name='17', max_length=255, blank=True, null=True)
    field_18 = models.CharField(verbose_name='18', max_length=255, blank=True, null=True)
    field_19 = models.CharField(verbose_name='19', max_length=255, blank=True, null=True)
    field_20 = models.CharField(verbose_name='20', max_length=255, blank=True, null=True)

    def __str__(self):
        return f"{self.thread_id}"


# 扩展字段表。用于声明扩展字段数据表中的(有序)字段具体对应的什么键名。注意：扩展字段是对分类的扩展，而不是主类别的扩展
class ThreadExtendField(models.Model):
    class Meta:
        db_table = 'thread_extend_field'
        verbose_name_plural = '扩展字段表'
        unique_together = (("classify_id", "field"),)  # 组合唯一，分类+字段

    id = models.AutoField(verbose_name='信息ID', primary_key=True)

    # 数据库生成classify_id字段
    classify_id = models.ForeignKey(verbose_name='分类ID', to=ThreadClassify, db_column='classify_id', related_name='+',
                                    on_delete=models.DO_NOTHING)
    field = models.CharField(verbose_name='字段', max_length=255)  # 眏射ThreadExtendData表的键名
    extend_field = models.CharField(verbose_name='映射字段', max_length=255)  # 眏射ThreadExtendData表的键名
    value = models.CharField(verbose_name='内容', max_length=255, null=True, blank=True)
    type = models.CharField(verbose_name='类型', max_length=255, blank=True, null=True)
    unit = models.CharField(verbose_name='参数单位', max_length=255, blank=True, null=True)

    def __str__(self):
        return f"{self.id}"


# 信息表，即主表
class Thread(models.Model):
    class Meta:
        ordering = ['-create_time']
        db_table = 'thread'  # 指定数据库的表名，否则默认会显示app名+class名。
        verbose_name_plural = '1. 信息表'  # 指定管理界面的别名，否则默认显示class名。末尾不加s。

    # ID 默认自增
    id = models.BigAutoField(verbose_name='ID', primary_key=True)
    is_deleted = models.BooleanField(verbose_name='是否删除', default=False)
    category_id = models.ForeignKey(verbose_name='类别ID', to=ThreadCategory, db_column='category_id', related_name='+',
                                    on_delete=models.DO_NOTHING, default=1)
    classify_id = models.ForeignKey(verbose_name='分类ID', to=ThreadClassify, db_column='classify_id', related_name='+',
                                    on_delete=models.DO_NOTHING, null=True, blank=True)
    show_id = models.ForeignKey(verbose_name='展示ID', to=ThreadShow, db_column='show_id', related_name='+',
                                on_delete=models.DO_NOTHING, null=True, blank=True,
                                default=1)  # 如果没有传入显示类型，则使用分类表中的默认显示类型
    # user_id = models.ForeignKey(verbose_name='用户ID', to=User, db_column='user_id', related_name='+', on_delete=models.DO_NOTHING)  # 登录后，自动填。
    user_id = models.BigIntegerField(verbose_name='用户ID', db_index=True)  # 登录后，自动填。
    auth_id = models.ForeignKey(verbose_name='权限ID', to=ThreadAuth, db_column='auth_id', related_name='+',
                                on_delete=models.DO_NOTHING, default=1)

    title = models.CharField(verbose_name='标题', max_length=200, blank=True, null=True, db_index=True)
    content = UEditorField(verbose_name='内容', blank=True, null=True)
    # content = models.TextField(verbose_name='内容', blank=True, null=True)
    summary = models.CharField(verbose_name='任务摘要', max_length=150, blank=True, null=True, default="")
    ip = models.GenericIPAddressField(verbose_name='IP地址', protocol='both',
                                      default=socket.gethostbyname(socket.gethostname()))  # 只记录创建时的IP

    has_enroll = models.BooleanField(verbose_name='开启报名', default=False)
    has_fee = models.BooleanField(verbose_name='开启小费', default=False)
    has_comment = models.BooleanField(verbose_name='开启评论', default=True)

    cover = models.CharField(verbose_name='封面', max_length=1024, blank=True, null=True)
    video = models.CharField(verbose_name='视频', max_length=1024, blank=True, null=True)
    photos = models.JSONField(verbose_name='照片集', blank=True, null=True)  # 对象数组，存放{id, url} 获取列表时使用，查看详细时再匹配资源表
    files = models.JSONField(verbose_name='文件集', blank=True, null=True)  # 对象数组，存放{id, url}

    price = models.DecimalField(verbose_name='价格', max_digits=32, decimal_places=8, db_index=True, null=True,
                                blank=True)  # add-2022-05-20
    author = models.CharField(verbose_name='作者', max_length=255, blank=True, null=True)  # add-2022-05-20
    is_original = models.BooleanField(verbose_name='是否原创', default=True)  # add-2022-05-20

    create_time = models.DateTimeField(verbose_name='创建时间', default=timezone.now)
    update_time = models.DateTimeField(verbose_name='更新时间', auto_now=True)  # 不显示，系统自动填。

    # 用户的修改记录等日志信息，数组对象类型 使用CRC32来比较哪些字段被修改过，并记录
    logs = models.JSONField(verbose_name='日志', blank=True, null=True, default=list)
    more = models.JSONField(verbose_name='更多信息', blank=True, null=True)

    def __str__(self):
        return f"{self.id}"

    # 判断指定字段长度,超出部分用省略号代替
    def short_content(self):
        if len(str(self.content)) > 50:
            return '{}...'.format(str(self.content)[0:50])
        else:
            return str(self.content)

    # 字段数据处理后,字段verbose_name参数失效
    # 需要重新指定,否则列表页字段名显示的是方法名(short_content)
    short_content.short_description = '内容'

    def short_cover(self):
        if len(str(self.cover)) > 15:
            return '{}...'.format(str(self.cover)[0:15])
        else:
            return str(self.cover)

    short_cover.short_description = '封面'

    def short_video(self):
        if len(str(self.video)) > 15:
            return '{}...'.format(str(self.video)[0:15])
        else:
            return str(self.video)

    short_video.short_description = '视频'

    def short_photos(self):
        if len(str(self.photos)) > 15:
            return '{}...'.format(str(self.photos)[0:15])
        else:
            return str(self.photos)

    short_photos.short_description = '照片集'

    def short_files(self):
        if len(str(self.files)) > 15:
            return '{}...'.format(str(self.files)[0:15])
        else:
            return str(self.files)

    short_files.short_description = '文件集'

    def short_logs(self):
        if len(str(self.logs)) > 15:
            return '{}...'.format(str(self.logs)[0:15])
        else:
            return str(self.logs)

    short_logs.short_description = '日志'

    def short_more(self):
        if len(str(self.more)) > 30:
            return '{}...'.format(str(self.more)[0:30])
        else:
            return str(self.logs)

    short_more.short_description = '更多信息'


class ThreadStatistic(models.Model):
    thread_id = models.OneToOneField(Thread, verbose_name='信息主表', on_delete=models.DO_NOTHING, related_name='statistic',
                                     null=True, blank=True)
    flag_classifies = models.CharField(verbose_name='分类**', max_length=255, null=True, blank=True)
    flag_weights = models.CharField(verbose_name='权重**', max_length=255, null=True, blank=True)
    weight = models.FloatField(verbose_name='权重', default=0, db_index=True)
    views = models.IntegerField(verbose_name='浏览数', default=0)
    plays = models.IntegerField(verbose_name='完阅数', default=0)
    comments = models.IntegerField(verbose_name='评论数', default=0)
    likes = models.IntegerField(verbose_name='点赞数', default=0)
    favorite = models.IntegerField(verbose_name='收藏数', default=0)
    shares = models.IntegerField(verbose_name='分享数', default=0)

    class Meta:
        db_table = 'thread_statistic'
        verbose_name = '信息统计表'
        verbose_name_plural = verbose_name


# 标签类型，存放预置标签。用于智能化推送信息，以及关键字检索。未来应设计成根据信息内容自动生成标签。
class ThreadTag(models.Model):
    class Meta:
        db_table = 'thread_tag'
        verbose_name_plural = '标签类型表'

    id = models.AutoField(verbose_name='ID', primary_key=True)
    value = models.CharField(verbose_name='标签名', max_length=255, blank=True, null=True)
    thread = models.ManyToManyField(to='Thread', through='ThreadTagMapping', through_fields=('tag_id', 'thread_id'),
                                    blank=True)

    def __str__(self):
        return f"{self.value}"


# 标签映射，存放数据。即将标签和信息关联起来
class ThreadTagMapping(models.Model):
    class Meta:
        db_table = 'thread_tag_mapping'
        verbose_name_plural = '标签映射表'

    id = models.AutoField(verbose_name='ID', primary_key=True)
    thread_id = models.ForeignKey(verbose_name='信息ID', to=Thread, db_column='thread_id', related_name='+',
                                  on_delete=models.DO_NOTHING)
    tag_id = models.ForeignKey(verbose_name='标签ID', to=ThreadTag, db_column='tag_id', related_name='+',
                               on_delete=models.DO_NOTHING)

    def __str__(self):
        return f"{self.id}"


# 图片权限。作者可以指定上传的图片的访问权限。如公开照片、阅后即焚、已焚毁、红包、红包阅后即焚、红包阅后已焚毁
class ThreadImageAuth(models.Model):
    class Meta:
        db_table = 'thread_image_auth'
        verbose_name_plural = '图片权限表'

    id = models.AutoField(verbose_name='ID', primary_key=True)
    value = models.CharField(verbose_name='值', max_length=255, blank=True, null=True)


# 图片信息表。用于存放图片的各种信息，存放图片地址但不存放图。
class ThreadResource(models.Model):
    class Meta:
        db_table = 'thread_resource'
        verbose_name_plural = '图片表'

    id = models.BigIntegerField(verbose_name='ID', primary_key=True)
    name = models.CharField(verbose_name='图片名称', max_length=255, null=True, blank=True)
    url = models.CharField(verbose_name='图片链接', max_length=1024, null=True, blank=True)
    filename = models.CharField(verbose_name='文件名', max_length=255, null=True, blank=True)
    filetype = models.SmallIntegerField(verbose_name='文件类型', null=True, blank=True)  # 文件类型0:图片，1:视频，2:文件
    format = models.CharField(verbose_name='文件格式', max_length=50)
    image_auth_id = models.ForeignKey(verbose_name='图片权限ID', to=ThreadImageAuth, db_column='image_auth_id',
                                      related_name='+', on_delete=models.DO_NOTHING, null=True, blank=True)
    price = models.DecimalField(verbose_name='价格', max_digits=32, decimal_places=8, db_index=True, null=True,
                                blank=True)
    snapshot = models.JSONField(verbose_name='快照', blank=True, null=True)  # 存放图片的快照数据，如缩略图等。json对象
    logs = models.JSONField(verbose_name='日志', blank=True, null=True)  # 用于存放点击量，点赞量等,数组对象
    # user_id = models.ForeignKey(verbose_name='用户ID', to=User, db_column='user_id', related_name='+', on_delete=models.DO_NOTHING)
    user_id = models.BigIntegerField(verbose_name='用户ID', )
    thread = models.ManyToManyField(to='Thread', through='ThreadToResource',
                                    through_fields=('resource_id', 'thread_id'), blank=True)


# 标签映射，存放数据。即将标签和信息关联起来
class ThreadToResource(models.Model):
    class Meta:
        db_table = 'thread_to_resource'
        verbose_name_plural = '图文关联表'

    id = models.AutoField(verbose_name='ID', primary_key=True)
    thread_id = models.ForeignKey(verbose_name='信息ID', to=Thread, db_column='thread_id', related_name='+',
                                  on_delete=models.DO_NOTHING)
    resource_id = models.ForeignKey(verbose_name='图片ID', to=ThreadResource, db_column='resource_id', related_name='+',
                                    on_delete=models.DO_NOTHING)
